@@ -124,7 +124,7 @@ MediaView::MediaView(QWidget *parent) : QWidget(parent) {
 #ifdef APP_DEMO
     demoTimer = new QTimer(this);
     demoTimer->setSingleShot(true);
-    demoTimer->setInterval(30000);
+    demoTimer->setInterval(60000);
     connect(demoTimer, SIGNAL(timeout()), SLOT(demoMessage()));
 #endif
 
@@ -425,7 +425,7 @@ void MediaView::startPlaying() {
     }
 
 #ifdef APP_DEMO
-    demoTimer->start();
+    demoTimer->start(30000);
 #endif
 
 }
@@ -464,7 +464,9 @@ void MediaView::skip() {
 }
 
 void MediaView::playbackFinished() {
-    if (mediaObject->currentTime() < mediaObject->totalTime()) {
+    // qDebug() << "finished" << mediaObject->currentTime() << mediaObject->totalTime();
+    // add 10 secs for imprecise Phonon backends (VLC, Xine)
+    if (mediaObject->currentTime() + 10000 < mediaObject->totalTime()) {
         // mediaObject->seek(mediaObject->currentTime());
         QTimer::singleShot(3000, this, SLOT(playbackResume()));
     } else skip();
@@ -607,7 +609,7 @@ void MediaView::demoMessage() {
         QDesktopServices::openUrl(QString(Constants::WEBSITE) + "#download");
     } else {
         mediaObject->play();
-        demoTimer->start();
+        demoTimer->start(300000);
     }
 }
 #endif
